package jms.cluster.ejb;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Properties;

import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.ejb.MessageDrivenContext;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.naming.InitialContext;

import org.jboss.ejb3.annotation.ResourceAdapter;


@MessageDriven(activationConfig = {
		@ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue"),
		@ActivationConfigProperty(propertyName = "destination", propertyValue = "queue/requestQ"),
		@ActivationConfigProperty(propertyName = "acknowledgeMode", propertyValue = "Auto-acknowledge") })
@ResourceAdapter("hornetq-ra.rar")
@TransactionManagement(value = TransactionManagementType.CONTAINER)
@TransactionAttribute(value = TransactionAttributeType.REQUIRED)


public class MDB1 implements MessageListener {
	private static final long serialVersionUID = 1L;

	@Resource
	MessageDrivenContext ctx;

	@Resource(mappedName = "java:PrimaryMDBJmsXA")
	private ConnectionFactory connectionFactory;

	@Resource(mappedName = "java:PrimaryAlternateJmsXA")
	private ConnectionFactory altConnectionFactory;

	/**
	 * Creates a new transacted listener consumer.
	 */
	public ClusteredQueueEJBListener() {
	}

	public void onMessage(Message message) {
		try {
			TextMessage txtMessage = (TextMessage) message;
			System.out.println("Consumed by mdb 1 : " + txtMessage.getText());
			Thread.sleep(30000);
			sendResponse(txtMessage.getText());
			System.out.println("onMessage -> Sent response");
		} catch (Exception e) {
			System.out.println("onMessage -> in exception catch block");
			ctx.setRollbackOnly();
			System.out
					.println("Proceeding after exception in exception catch block...");
		}
	}

	private void sendResponse(String message) throws Exception {
		// Step 5. Create a connection to a remote server using a
		// connection-factory (look at the deployed file jms-remote-ds.xml)
		System.out.println("0. About to send the response message : "
				+ connectionFactory);
		Connection conn1 = null;
		Connection conn2 = null;
		try {
			conn1 = connectionFactory.createConnection();
			sendMessage(conn1, message);
		} catch (JMSException e) {
			System.out
					.println("sendResponse -> Problem getting connection to primary server, attempting 2ndry");
			try {
				conn2 = altConnectionFactory.createConnection();
				System.out
						.println("sendResponse -> Got connection to secondary server..........");
				sendMessage(conn2, message);
			} catch (JMSException e2) {
				System.out
						.println("sendResponse -> Error getting secondary connection factory "
								+ e.getMessage());
				e2.printStackTrace();
				throw e;
			}
		} catch (Exception e) {
			System.out.println("sendResponse -> In exception block");
			e.printStackTrace();
			throw e;
		} finally {
			// Step 7. Close the connection. (Since this is a JCA connection,
			// this will just place the connection back to a connection pool)
			try {
				if (conn1 != null) {
					conn1.close();
				}
				if (conn2 != null) {
					conn2.close();
				}
			} catch (JMSException e) {
				e.printStackTrace();
			}
		}
	}

	private void sendMessage(Connection myConn, String message)
			throws JMSException {
		// Step 6. Send a message to a QueueA on the remote server
		Session sess = myConn.createSession(false, Session.AUTO_ACKNOWLEDGE);
		System.out.println("2. created session");

		Queue destQueueA = sess.createQueue("responseQ");

		MessageProducer prodA = sess.createProducer(destQueueA);

		prodA.send(sess.createTextMessage(message));
		System.out.println("Sent message \"" + message + "\" to responseQ");
	}

}